@@ -23,7 +23,7 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	1 March 2006
- * Last Edited:	18 January 2007
+ * Last Edited:	21 June 2007
  */
 
 
@@ -89,7 +89,6 @@ typedef struct mix_local {
   int msgfd;			/* file description of current msg file */
   int mfd;			/* file descriptor of open metadata */
   unsigned long metaseq;	/* metadata sequence */
-  char *dir;			/* mailbox directory name */
   char *index;			/* mailbox index name */
   unsigned long indexseq;	/* index sequence */
   char *status;			/* mailbox status name */
@@ -99,7 +98,6 @@ typedef struct mix_local {
   unsigned char *buf;		/* temporary buffer */
   unsigned long buflen;		/* current size of temporary buffer */
   unsigned int expok : 1;	/* non-zero if expunge reports OK */
-  unsigned int burpneeded : 1;	/* burp needed */
   unsigned int internal : 1;	/* internally opened, do not validate */
 } MIXLOCAL;
 
@@ -163,13 +161,13 @@ long mix_append_msg (MAILSTREAM *stream,FILE *f,char *flags,MESSAGECACHE *delt,
 FILE *mix_parse (MAILSTREAM *stream,FILE **idxf,long iflags,long flags);
 char *mix_meta_slurp (MAILSTREAM *stream,unsigned long *seq);
 long mix_meta_update (MAILSTREAM *stream);
-long mix_index_update (MAILSTREAM *stream,FILE **idxf,long flag);
+long mix_index_update (MAILSTREAM *stream,FILE *idxf);
 long mix_status_update (MAILSTREAM *stream,FILE **statf,long flag);
 FILE *mix_data_open (MAILSTREAM *stream,int *fd,long *size,
 		     unsigned long newsize);
 FILE *mix_sortcache_open (MAILSTREAM *stream);
 long mix_sortcache_update (MAILSTREAM *stream,FILE **sortcache);
-char *mix_read_record (FILE *f,char *buf,unsigned long buflen);
+char *mix_read_record (FILE *f,char *buf,unsigned long buflen,char *type);
 long mix_read_sequence (FILE *f);
 char *mix_dir (char *dst,char *name);
 char *mix_file (char *dst,char *dir,char *name);
@@ -403,11 +401,14 @@ long mix_create (MAILSTREAM *stream,char *mailbox)
   DRIVER *test;
   FILE *f;
   int c,i;
-  char *s,*t,tmp[MAILTMPLEN],file[MAILTMPLEN];
+  char *t,tmp[MAILTMPLEN],file[MAILTMPLEN];
+  char *s = strrchr (mailbox,'/');
   unsigned long now = time (NIL);
   long ret = NIL;
+				/* always create \NoSelect if trailing /  */
+  if (s && !s[1]) return dummy_create (stream,mailbox);
 				/* validate name */
-  if (mix_dirfmttest ((s = strrchr (mailbox,'/')) ? s + 1 : mailbox))
+  if (mix_dirfmttest (s ? s + 1 : mailbox))
     sprintf(tmp,"Can't create mailbox %.80s: invalid MIX-format name",mailbox);
 				/* must not already exist */
   else if ((test = mail_valid (NIL,mailbox,NIL)) &&
@@ -439,16 +440,21 @@ long mix_create (MAILSTREAM *stream,char *mailbox)
     if (!dummy_create_path (stream,file,get_dir_protection (mailbox)))
       sprintf (tmp,"Can't create mix mailbox index: %.80s",strerror (errno));
     else {
+      set_mbx_protections (mailbox,file);
       strcpy (s,MIXSTATUS);	/* create status */
       if (!dummy_create_path (stream,file,get_dir_protection (mailbox)))
 	sprintf (tmp,"Can't create mix mailbox status: %.80s",
 		 strerror (errno));
       else {
+	set_mbx_protections (mailbox,file);
 	sprintf (s,"%08lx",now);/* message file */
 	if (!dummy_create_path (stream,file,get_dir_protection (mailbox)))
 	  sprintf (tmp,"Can't create mix mailbox data: %.80s",
 		   strerror (errno));
-	else ret = LONGT;	/* declare success at this point */
+	else {
+	  set_mbx_protections (mailbox,file);
+	  ret = LONGT;	/* declare success at this point */
+	}
       }
     }
   }
@@ -610,27 +616,35 @@ MAILSTREAM *mix_open (MAILSTREAM *stream)
 				/* make temporary buffer */
   LOCAL->buf = (char *) fs_get (CHUNKSIZE);
   LOCAL->buflen = CHUNKSIZE - 1;
-				/* get directory name */
-  LOCAL->dir = cpystr (mix_dir (LOCAL->buf,stream->mailbox));
-				/* open metadata file */
-  if (((LOCAL->mfd = open (mix_file (LOCAL->buf,LOCAL->dir,MIXMETA),
-			   stream->rdonly ? O_RDONLY : O_RDWR,NIL)) < 0) ||
-      flock (LOCAL->mfd,LOCK_SH)) {
+				/* set stream->mailbox to be directory name */
+  mix_dir (LOCAL->buf,stream->mailbox);
+  fs_give ((void **) &stream->mailbox);
+  stream->mailbox = cpystr (LOCAL->buf);
+  LOCAL->msgfd = -1;		/* currently no file open */
+  if (!(((!stream->rdonly &&	/* open metadata file */
+	  ((LOCAL->mfd = open (mix_file (LOCAL->buf,stream->mailbox,MIXMETA),
+			       O_RDWR,NIL)) >= 0)) ||
+	 ((stream->rdonly = T) &&
+	  ((LOCAL->mfd = open (mix_file (LOCAL->buf,stream->mailbox,MIXMETA),
+			       O_RDONLY,NIL)) >= 0))) &&
+	!flock (LOCAL->mfd,LOCK_SH))) {
     MM_LOG ("Error opening mix metadata file",ERROR);
     mix_close (stream,NIL);
     stream = NIL;		/* open fails */
   }
   else {			/* metadata open, complete open */
-    LOCAL->index = cpystr (mix_file (LOCAL->buf,LOCAL->dir,MIXINDEX));
-    LOCAL->status = cpystr (mix_file (LOCAL->buf,LOCAL->dir,MIXSTATUS));
-    LOCAL->sortcache = cpystr (mix_file (LOCAL->buf,LOCAL->dir,MIXSORTCACHE));
-    LOCAL->msgfd = -1;		/* currently no file open */
+    LOCAL->index = cpystr (mix_file (LOCAL->buf,stream->mailbox,MIXINDEX));
+    LOCAL->status = cpystr (mix_file (LOCAL->buf,stream->mailbox,MIXSTATUS));
+    LOCAL->sortcache = cpystr (mix_file (LOCAL->buf,stream->mailbox,
+					 MIXSORTCACHE));
     stream->sequence++;		/* bump sequence number */
 				/* parse mailbox */
     stream->nmsgs = stream->recent = 0;
     if (silent = stream->silent) LOCAL->internal = T;
     stream->silent = T;
     if (mix_ping (stream)) {	/* do initial ping */
+				/* try burping in case we are exclusive */
+      if (!stream->rdonly) mix_expunge (stream,"",NIL);
       if (!(stream->nmsgs || stream->silent))
 	MM_LOG ("Mailbox is empty",(long) NIL);
       stream->silent = silent;	/* now notify upper level */
@@ -659,12 +673,12 @@ void mix_close (MAILSTREAM *stream,long options)
   if (LOCAL) {			/* only if a file is open */
     int silent = stream->silent;
     stream->silent = T;		/* note this stream is dying */
-    if (options & CL_EXPUNGE) mix_expunge (stream,NIL,NIL);
+				/* checkpoint or expunge */
+    mix_expunge (stream,(options & CL_EXPUNGE) ? NIL : "",NIL);
 				/* close current message file if open */
     if (LOCAL->msgfd >= 0) close (LOCAL->msgfd);
 				/* close current metadata file if open */
     if (LOCAL->mfd >= 0) close (LOCAL->mfd);
-    if (LOCAL->dir) fs_give ((void **) &LOCAL->dir);
     if (LOCAL->index) fs_give ((void **) &LOCAL->index);
     if (LOCAL->status) fs_give ((void **) &LOCAL->status);
     if (LOCAL->sortcache) fs_give ((void **) &LOCAL->sortcache);
@@ -698,7 +712,7 @@ char *mix_header (MAILSTREAM *stream,unsigned long msgno,unsigned long *length,
 				/* is message in current message file? */
   if ((LOCAL->msgfd < 0) || (elt->private.spare.data != LOCAL->curmsg)) {
     if (LOCAL->msgfd >= 0) close (LOCAL->msgfd);
-    if ((LOCAL->msgfd = open (mix_file_data (LOCAL->buf,LOCAL->dir,
+    if ((LOCAL->msgfd = open (mix_file_data (LOCAL->buf,stream->mailbox,
 					     elt->private.spare.data),
 					     O_RDONLY,NIL)) < 0) return "";
 				/* got file */
@@ -761,7 +775,7 @@ long mix_text (MAILSTREAM *stream,unsigned long msgno,STRING *bs,long flags)
 				/* is message in current message file? */
   if ((LOCAL->msgfd < 0) || (elt->private.spare.data != LOCAL->curmsg)) {
     if (LOCAL->msgfd >= 0) close (LOCAL->msgfd);
-    if ((LOCAL->msgfd = open (mix_file_data (LOCAL->buf,LOCAL->dir,
+    if ((LOCAL->msgfd = open (mix_file_data (LOCAL->buf,stream->mailbox,
 					     elt->private.spare.data),
 					     O_RDONLY,NIL)) < 0) return NIL;
 				/* got file */
@@ -774,7 +788,8 @@ long mix_text (MAILSTREAM *stream,unsigned long msgno,STRING *bs,long flags)
     elt->seen = T;		/* mark as seen */
     MM_FLAGS (stream,elt->msgno);
     if (statf) {		/* update status file if possible */
-      elt->private.mod = LOCAL->statusseq = mix_modseq (LOCAL->statusseq);
+      if (!stream->rdonly)
+	elt->private.mod = LOCAL->statusseq = mix_modseq (LOCAL->statusseq);
       mix_status_update (stream,&statf,LONGT);
     }
     if (idxf) fclose (idxf);	/* release index file */
@@ -837,19 +852,19 @@ void mix_flag (MAILSTREAM *stream,char *sequence,char *flag,long flags)
 	    (old.flagged != elt->flagged) ||
 	    (old.answered != elt->answered) || (old.draft != elt->draft) ||
 	    (old.user_flags != elt->user_flags)) {
-	  elt->private.mod = LOCAL->statusseq = seq;
+	  if (!stream->rdonly) elt->private.mod = LOCAL->statusseq = seq;
 	  MM_FLAGS (stream,elt->msgno);
 	}
       }
-    if (statf) {		/* update and close status file */
+    if (statf) {		/* update and close status file after change */
       if (seq == LOCAL->statusseq) mix_status_update (stream,&statf,LONGT);
-      else fclose (statf);	/* no update, just close */
 				/* update metadata if created a keyword */
       if ((ffkey < NUSERFLAGS) && stream->user_flags[ffkey] &&
 	  !mix_meta_update (stream))
 	MM_LOG ("Error updating mix metadata after keyword creation",ERROR);
     }
   }
+  if (statf) fclose (statf);	/* release status file if still open */
   if (idxf) fclose (idxf);	/* release index file */
 }
 
@@ -986,19 +1001,23 @@ long mix_ping (MAILSTREAM *stream)
 
 void mix_check (MAILSTREAM *stream)
 {
-  mix_expunge (stream,"",NIL);	/* do an expunge of no messages */
+  if (stream->rdonly)		/* won't do on readonly files! */
+    MM_LOG ("Checkpoint ignored on readonly mailbox",NIL);
+				/* do an expunge of no messages */
+  if (mix_expunge (stream,"",NIL)) MM_LOG ("Check completed",(long) NIL);
 }
 
 /* MIX mail expunge mailbox
  * Accepts: MAIL stream
  *	    sequence to expunge if non-NIL
  *	    expunge options
- * Returns: T, always
+ * Returns: T on success, NIL if failure
  */
 
 long mix_expunge (MAILSTREAM *stream,char *sequence,long options)
 {
-  FILE *idxf,*statf;
+  FILE *statf;
+  FILE *idxf = NIL;
   MESSAGECACHE *elt;
   int ifd,sfd;
   long ret;
@@ -1007,119 +1026,111 @@ long mix_expunge (MAILSTREAM *stream,char *sequence,long options)
   unsigned long reclaimed = 0;
   int checkpoint = (sequence && !*sequence);
   LOCAL->expok = T;		/* expunge during ping is OK */
-  if (!(ret = checkpoint || !sequence || ((options & EX_UID) ?
-					  mail_uid_sequence (stream,sequence) :
-					  mail_sequence (stream,sequence))));
-  else if (stream->rdonly)	/* won't do on readonly files! */
-    MM_LOG (checkpoint ? "Checkpoint ignored on readonly mailbox" :
-	    "Expunge ignored on readonly mailbox",WARN);
-				/* read index and open status exclusive */
-  if (statf = mix_parse (stream,&idxf,LONGT,MP_VALID)) {
-    if (!checkpoint) {	       /* expunge unless just checkpointing */
-      for (i = 1; i <= stream->nmsgs;) {
-	elt = mail_elt (stream,i);
-				/* need to delete this message? */
-	if (sequence ? elt->sequence : elt->deleted) {
-	  mail_expunged (stream,i);
-	  ++nexp;		/* yes, make it so */
-	}
-	else ++i;		/* otherwise advance to next message */
+  if (!(ret = checkpoint || !sequence ||
+	((options & EX_UID) ?
+	 mail_uid_sequence (stream,sequence) :
+	 mail_sequence (stream,sequence))));
+  else if (!stream->rdonly &&	/* read index and open status exclusive */
+	   (statf = mix_parse (stream,&idxf,LONGT,MP_VALID))) {
+				/* expunge unless just checkpointing */
+    if (!checkpoint) for (i = 1; i <= stream->nmsgs;) {
+      elt = mail_elt (stream,i);/* need to expunge this message? */
+      if (sequence ? elt->sequence : elt->deleted) {
+	++nexp;			/* yes, make it so */
+	mail_expunged (stream,i);
       }
+      else ++i;		       /* otherwise advance to next message */
     }
 
-				/* need to burp? */
-    if (nexp || checkpoint || LOCAL->burpneeded) {
-				/* sole process with mailbox selected? */
-      if (flock (LOCAL->mfd,LOCK_EX|LOCK_NB)) LOCAL->burpneeded = T;
-      else {			/* got exclusive lock, can burp */
-	void *a;
-	struct direct **names = NIL;
-	long nfiles = scandir (LOCAL->dir,&names,mix_select,mix_msgfsort);
-	if (nfiles > 0) {	/* if have message files */
-	  MIXBURP *burp,*cur;
-	  LOCAL->burpneeded = NIL;
+				/* burp if can get exclusive access */
+    if (!flock (LOCAL->mfd,LOCK_EX|LOCK_NB)) {
+      void *a;
+      struct direct **names = NIL;
+      long nfiles = scandir (stream->mailbox,&names,mix_select,mix_msgfsort);
+      if (nfiles > 0) {		/* if have message files */
+	MIXBURP *burp,*cur;
 				/* initialize burp list */
-	  for (i = 0, burp = cur = NIL; i < nfiles; ++i) {
-	    MIXBURP *nxt = (MIXBURP *) memset (fs_get (sizeof (MIXBURP)),0,
-					       sizeof (MIXBURP));
+	for (i = 0, burp = cur = NIL; i < nfiles; ++i) {
+	  MIXBURP *nxt = (MIXBURP *) memset (fs_get (sizeof (MIXBURP)),0,
+					     sizeof (MIXBURP));
 				/* another file found */
-	    if (cur) cur = cur->next = nxt;
-	    else cur = burp = nxt;
-	    cur->name = names[i]->d_name;
-	    cur->fileno = strtoul (cur->name + sizeof (MIXNAME) - 1,NIL,16);
-	    cur->tail = &cur->set;
-	    fs_give ((void **) &names[i]);
-	  }
+	  if (cur) cur = cur->next = nxt;
+	  else cur = burp = nxt;
+	  cur->name = names[i]->d_name;
+	  cur->fileno = strtoul (cur->name + sizeof (MIXNAME) - 1,NIL,16);
+	  cur->tail = &cur->set;
+	  fs_give ((void **) &names[i]);
+	}
 				/* now load ranges */
-	  for (i = 1, cur = burp; ret && (i <= stream->nmsgs); i++) {
+	for (i = 1, cur = burp; ret && (i <= stream->nmsgs); i++) {
 				/* is this message in current set? */
-	    elt = mail_elt (stream,i);
-	    if (cur && (elt->private.spare.data != cur->fileno)) {
+	  elt = mail_elt (stream,i);
+	  if (cur && (elt->private.spare.data != cur->fileno)) {
 				/* restart if necessary */
-	      if (elt->private.spare.data < cur->fileno) cur = burp;
+	    if (elt->private.spare.data < cur->fileno) cur = burp;
 				/* hunt for appropriate mailbox */
-	      while (cur && (elt->private.spare.data > cur->fileno))
-		cur = cur->next;
+	    while (cur && (elt->private.spare.data > cur->fileno))
+	      cur = cur->next;
 				/* ought to have found it now... */
-	      if (cur && (elt->private.spare.data != cur->fileno)) cur = NIL;
-	    }
+	    if (cur && (elt->private.spare.data != cur->fileno)) cur = NIL;
+	  }
 				/* if found, add to set */
-	    if (cur) ret = mix_addset(&cur->tail,elt->private.special.offset,
-				      elt->private.msg.header.offset +
-				      elt->rfc822_size);
-	    else {		/* uh-oh */
-	      sprintf (LOCAL->buf,"Can't locate mix message file %.08x",
-		       elt->private.spare.data);
-	      MM_LOG (LOCAL->buf,ERROR);
-	      ret = NIL;
-	    }
+	  if (cur) ret = mix_addset (&cur->tail,elt->private.special.offset,
+				     elt->private.msg.header.offset +
+				     elt->rfc822_size);
+	  else {		/* uh-oh */
+	    sprintf (LOCAL->buf,"Can't locate mix message file %.08x",
+		     elt->private.spare.data);
+	    MM_LOG (LOCAL->buf,ERROR);
+	    ret = NIL;
 	  }
-
-	  if (ret) {		/* proceed only if no errors */
-				/* burp all files */
-	    for (cur = burp; ret && cur; cur = cur->next) {
+	}
+	if (ret) 		/* if no errors, burp all files */
+	  for (cur = burp; ret && cur; cur = cur->next) {
 				/* if non-empty, burp it */
-	      if (cur->set.last) ret = mix_burp (stream,cur,&reclaimed);
+	    if (cur->set.last) ret = mix_burp (stream,cur,&reclaimed);
 				/* empty, delete it unless new msg file */
-	      else if (mix_file_data (LOCAL->buf,LOCAL->dir,cur->fileno) &&
-		       ((cur->fileno == LOCAL->newmsg) ?
-			truncate (LOCAL->buf,0) : unlink (LOCAL->buf))) {
-		sprintf (LOCAL->buf,
-			 "Can't delete empty message file %.80s: %.80s",
-			 cur->name,strerror (errno));
-		MM_LOG (LOCAL->buf,WARN);
-	      }
+	    else if (mix_file_data (LOCAL->buf,stream->mailbox,cur->fileno) &&
+		     ((cur->fileno == LOCAL->newmsg) ?
+		      truncate (LOCAL->buf,0) : unlink (LOCAL->buf))) {
+	      sprintf (LOCAL->buf,
+		       "Can't delete empty message file %.80s: %.80s",
+		       cur->name,strerror (errno));
+	      MM_LOG (LOCAL->buf,WARN);
 	    }
 	  }
-	}
-	else MM_LOG ("No mix message files found during expunge",WARN);
-				/* free directory list */
-	if (a = (void *) names) fs_give ((void **) &a);
       }
+      else MM_LOG ("No mix message files found during expunge",WARN);
+				/* free directory list */
+      if (a = (void *) names) fs_give ((void **) &a);
+    }
+
 				/* either way, re-acquire shared lock */
-      flock (LOCAL->mfd,LOCK_SH);
-      /* Do this step even if ret is NIL (meaning some burp problem)! */
-      if (nexp || reclaimed) {	/* rewrite index and status if changed */
-	LOCAL->indexseq = mix_modseq (LOCAL->indexseq);
-	if (mix_index_update (stream,&idxf,LONGT)) {
-	  LOCAL->statusseq = mix_modseq (LOCAL->statusseq);
+    if (flock (LOCAL->mfd,LOCK_SH|LOCK_NB))
+      fatal ("Unable to re-acquire metadata shared lock!");
+    /* Do this step even if ret is NIL (meaning some burp problem)! */
+    if (nexp || reclaimed) {	/* rewrite index and status if changed */
+      LOCAL->indexseq = mix_modseq (LOCAL->indexseq);
+      if (mix_index_update (stream,idxf)) {
+	LOCAL->statusseq = mix_modseq (LOCAL->statusseq);
 				/* set failure if update fails */
-	  ret = mix_status_update (stream,&statf,LONGT);
-	}
+	ret = mix_status_update (stream,&statf,LONGT);
       }
     }
     if (statf) fclose (statf);	/* close status if still open */
   }
   if (idxf) fclose (idxf);	/* close index if still open */
-  if (ret) {			/* don't change message if error */
-    if (nexp) sprintf (LOCAL->buf,"Expunged %lu messages",nexp);
-    else if (reclaimed)
-      sprintf (LOCAL->buf,"Reclaimed %lu bytes of expunged space",reclaimed);
-    else if (checkpoint) strcpy (LOCAL->buf,"Check completed");
-    else strcpy (LOCAL->buf,"No messages deleted, so no update needed");
-    MM_LOG (LOCAL->buf,(long) NIL);
-  }
   LOCAL->expok = NIL;		/* cancel expok */
+  if (ret) {			/* only if success */
+    char *s;
+    if (nexp) sprintf (s = LOCAL->buf,"Expunged %lu messages",nexp);
+    else if (reclaimed)
+      sprintf (s=LOCAL->buf,"Reclaimed %lu bytes of expunged space",reclaimed);
+    else s = checkpoint ? NIL :
+      (stream->rdonly ? "Expunge ignored on readonly mailbox" :
+       "No messages deleted, so no update needed");
+    if (s) MM_LOG (s,(long) NIL);
+  }
   return ret;
 }
 
@@ -1204,7 +1215,7 @@ long mix_burp (MAILSTREAM *stream,MIXBURP *burp,unsigned long *reclaimed)
   unsigned long i;
   long ret = NIL;
 				/* build file name */
-  mix_file_data (LOCAL->buf,LOCAL->dir,burp->fileno);
+  mix_file_data (LOCAL->buf,stream->mailbox,burp->fileno);
 				/* need to burp at start or multiple ranges? */
   if (!burp->set.first && !burp->set.next) {
 				/* easy case, single range at start of file */
@@ -1394,7 +1405,8 @@ long mix_copy (MAILSTREAM *stream,char *sequence,char *mailbox,long options)
 	  if ((LOCAL->msgfd < 0) ||
 	      (elt->private.spare.data != LOCAL->curmsg)) {
 	    if (LOCAL->msgfd >= 0) close (LOCAL->msgfd);
-	    if ((LOCAL->msgfd = open (mix_file_data (LOCAL->buf,LOCAL->dir,
+	    if ((LOCAL->msgfd = open (mix_file_data (LOCAL->buf,
+						     stream->mailbox,
 						     elt->private.spare.data),
 				      O_RDONLY,NIL)) >= 0)
 	      LOCAL->curmsg = elt->private.spare.data;
@@ -1431,16 +1443,16 @@ long mix_copy (MAILSTREAM *stream,char *sequence,char *mailbox,long options)
 				/* finish write if success */
       if (ret && (ret = !fflush (msgf))) {
 	fclose (msgf);		/* all good, close the msg file now */
-				/* write new metadata, index, and status */
-	local->metaseq = local->indexseq = local->statusseq = seq;
+	if (!stream->rdonly)	/* write new metadata, index, and status */
+	  local->metaseq = local->indexseq = local->statusseq = seq;
 	if (ret = (mix_meta_update (astream) &&
-		   mix_index_update (astream,&idxf,LONGT))) {
+		   mix_index_update (astream,idxf))) {
 				/* success, delete if doing a move */
 	  if (options & CP_MOVE)
 	    for (i = 1; i <= stream->nmsgs; i++)
 	      if ((elt = mail_elt (stream,i))->sequence) {
 		elt->deleted = T;
-		elt->private.mod = LOCAL->statusseq = seq;
+		if (!stream->rdonly) elt->private.mod = LOCAL->statusseq = seq;
 		MM_FLAGS (stream,elt->msgno);
 	      }
 				/* done with status file now */
@@ -1556,10 +1568,10 @@ long mix_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
 				/* finish write if success */
 	if (ret && (ret = !fflush (msgf))) {
 	  fclose (msgf);	/* all good, close the msg file now */
-				/* write new metadata, index, and status */
-	  local->metaseq = local->indexseq = local->statusseq = seq;
+	  if (!stream->rdonly)	/* write new metadata, index, and status */
+	    local->metaseq = local->indexseq = local->statusseq = seq;
 	  if ((ret = (mix_meta_update (astream) &&
-		      mix_index_update (astream,&idxf,LONGT) &&
+		      mix_index_update (astream,idxf) &&
 		      mix_status_update (astream,&statf,LONGT))) && au) {
 	      (*au) (mailbox,astream->uid_validity,dst);
 	      dst = NIL;	/* don't free this set now */
@@ -1609,7 +1621,9 @@ long mix_append_msg (MAILSTREAM *stream,FILE *f,char *flags,MESSAGECACHE *delt,
   MESSAGECACHE *elt;
   int c,cs;
   unsigned long i,j,k,uf,hoff;
-  long sf = mail_parse_flags (stream,flags,&uf);
+  long sf;
+  stream->kwd_create = NIL;	/* don't copy unknown keywords */
+  sf = mail_parse_flags (stream,flags,&uf);
 				/* swell the cache */
   mail_exists (stream,++stream->nmsgs);
 				/* assign new UID from metadata */
@@ -1696,7 +1710,7 @@ FILE *mix_parse (MAILSTREAM *stream,FILE **idxf,long iflags,long flags)
 {
   int fd;
   unsigned long i,j;
-  char *s;
+  char *s,*t;
   struct stat sbuf;
   FILE *statf = NIL;
   short metarepairneeded = 0;
@@ -1788,7 +1802,8 @@ FILE *mix_parse (MAILSTREAM *stream,FILE **idxf,long iflags,long flags)
 				/* start with no messages */
       curfile = curfilesize = curpos = nmsgs = 0;
       LOCAL->indexseq = i;	/* update sequence, get first elt */
-      while ((s = mix_read_record (*idxf,LOCAL->buf,LOCAL->buflen)) && *s)
+      while ((s = mix_read_record (*idxf,LOCAL->buf,LOCAL->buflen,"index")) &&
+	     *s)
 	switch (*s) {
 	case ':':		/* message record */
 	  if (!(isxdigit (*++s) && (uid = strtoul (s,&t,16)))) msg = "UID";
@@ -1912,7 +1927,7 @@ FILE *mix_parse (MAILSTREAM *stream,FILE **idxf,long iflags,long flags)
 			  }
 			}
 			else {	/* new file, restart */
-			  if (stat (mix_file_data (LOCAL->buf,LOCAL->dir,
+			  if (stat (mix_file_data (LOCAL->buf,stream->mailbox,
 						   curfile = file),&sbuf)) {
 			    sprintf (tmp,"Missing mix data file: %.500s",
 				     LOCAL->buf);
@@ -1973,7 +1988,7 @@ FILE *mix_parse (MAILSTREAM *stream,FILE **idxf,long iflags,long flags)
 
 				/* repair metadata and index if needed */
     if ((metarepairneeded ? mix_meta_update (stream) : T) &&
-	(indexrepairneeded ? mix_index_update (stream,idxf,NIL) : T)) {
+	(indexrepairneeded ? mix_index_update (stream,*idxf) : T)) {
       MESSAGECACHE *elt;
       int fd;
       unsigned long uid,uf,sf,mod;
@@ -1993,7 +2008,8 @@ FILE *mix_parse (MAILSTREAM *stream,FILE **idxf,long iflags,long flags)
       else if (!stream->nmsgs);	/* do nothing if mailbox empty */
 				/* get sequence */
       else if (!(i = mix_read_sequence (statf)) || (i < LOCAL->statusseq)) {
-	sprintf (LOCAL->buf,"Error in mix status sequence record, i=%lu, seq=%lu",
+	sprintf (LOCAL->buf,
+		 "Error in mix status sequence record, i=%lu, seq=%lu",
 		 i,LOCAL->statusseq);
 	MM_LOG (LOCAL->buf,ERROR);
       }
@@ -2003,8 +2019,9 @@ FILE *mix_parse (MAILSTREAM *stream,FILE **idxf,long iflags,long flags)
 	elt = mail_elt (stream,i = 1);
 
 				/* read message records */
-	while ((s = mix_read_record (statf,LOCAL->buf,LOCAL->buflen)) && *s &&
-	       (*s++ == ':') && isxdigit (*s)) {
+	while ((t = s = mix_read_record (statf,LOCAL->buf,LOCAL->buflen,
+					 "status")) && *s && (*s++ == ':') &&
+	       isxdigit (*s)) {
 	  uid = strtoul (s,&s,16);
 	  if ((*s++ == ':') && isxdigit (*s)) {
 	    uf = strtoul (s,&s,16);
@@ -2052,15 +2069,16 @@ FILE *mix_parse (MAILSTREAM *stream,FILE **idxf,long iflags,long flags)
 	  break;		/* error somewhere */
 	}
 
-	if (!s || *s) {		/* error detected? */
-				/* non-null means bogus record */
-	  if (s) MM_LOG ("Error in mix status file message record",ERROR);
-	  fclose (statf);	/* either way, must punt */
-	  statf = NIL;
-	}
-	else if (updatep) {	/* did we see recent messages? */
-				/* yes, update status file now */
-	  LOCAL->statusseq = mix_modseq (LOCAL->statusseq);
+				/* error seen, or update needed? */
+	if (!t || *t || updatep) {
+	  if (t && *t) {	/* non-null means bogus record */
+	    char msg[MAILTMPLEN];
+	    sprintf (msg,"Error in mix status file message record%s: %.80s",
+		     writep ? ", fixing" : "",t);
+	    MM_LOG (msg,WARN);
+	  }
+	  if (!stream->rdonly)	/* either way, update status file */
+	    LOCAL->statusseq = mix_modseq (LOCAL->statusseq);
 	  if (writep) mix_status_update (stream,&statf,NIL);
 	}
       }
@@ -2135,7 +2153,7 @@ long mix_meta_update (MAILSTREAM *stream)
      * of NUSERFLAGS, MAXUSERFLAG or CHUNKSIZE, be sure to recalculate the
      * above assertation.
      */
-    sprintf (LOCAL->buf,SEQFMT,LOCAL->metaseq);
+    sprintf (LOCAL->buf,SEQFMT,LOCAL->metaseq = mix_modseq (LOCAL->metaseq));
     sprintf (LOCAL->buf + strlen (LOCAL->buf),MTAFMT,
 	     stream->uid_validity,stream->uid_last,LOCAL->newmsg);
     for (i = 0, c = 'K', s = ss = LOCAL->buf + strlen (LOCAL->buf);
@@ -2163,26 +2181,24 @@ long mix_meta_update (MAILSTREAM *stream)
 
 /* MIX update and close index
  * Accepts: MAIL stream
- *	    pointer to open FILE
- *	    close when done flag
+ *	    open FILE
  * Returns: T on success, NIL if error
  */
 
-long mix_index_update (MAILSTREAM *stream,FILE **idxf,long flag)
+long mix_index_update (MAILSTREAM *stream,FILE *idxf)
 {
-  FILE *f = *idxf;
   unsigned long i;
   long ret = LONGT;
   if (!stream->rdonly) {	/* do nothing if stream readonly */
-    rewind (f);			/* let's start at the very beginning */
-    ftruncate (fileno (f),0);	/* ...a very good place to start */
+    rewind (idxf);	       /* let's start at the very beginning */
+    ftruncate (fileno (idxf),0);
 				/* write modseq first */
-    fprintf (f,SEQFMT,LOCAL->indexseq);
+    fprintf (idxf,SEQFMT,LOCAL->indexseq);
 				/* then write all messages */
     for (i = 1; ret && (i <= stream->nmsgs); i++) {
       MESSAGECACHE *elt = mail_elt (stream,i);
 				/* write message index record */
-      fprintf (f,IXRFMT,elt->private.uid,
+      fprintf (idxf,IXRFMT,elt->private.uid,
 	       elt->year + BASEYEAR,elt->month,elt->day,
 	       elt->hours,elt->minutes,elt->seconds,
 	       elt->zoccident ? '-' : '+',elt->zhours,elt->zminutes,
@@ -2190,18 +2206,15 @@ long mix_index_update (MAILSTREAM *stream,FILE **idxf,long flag)
 	       elt->private.special.offset,
 	       elt->private.msg.header.offset,
 	       elt->private.msg.header.text.size);
-      if (ferror (f)) {
+      if (ferror (idxf)) {
 	MM_LOG ("Error updating mix index file",ERROR);
 	ret = NIL;
       }
     }
   }
-  if (flag) {			/* close index if requested */
-    if (fclose (f)) {
-      MM_LOG ("Error closing mix index file",ERROR);
-      ret = NIL;
-    }
-    *idxf = NIL;		/* either way, index is closed */
+  if (fflush (idxf)) {
+    MM_LOG ("Error flushing mix index file",ERROR);
+    ret = NIL;
   }
   return ret;
 }
@@ -2279,7 +2292,7 @@ FILE *mix_data_open (MAILSTREAM *stream,int *fd,long *size,
   unsigned long curend = (elt && (elt->private.spare.data == LOCAL->newmsg)) ?
     elt->private.special.offset + elt->private.msg.header.offset +
     elt->rfc822_size : 0;
-  if ((*fd = open (mix_file_data (LOCAL->buf,LOCAL->dir,LOCAL->newmsg),
+  if ((*fd = open (mix_file_data (LOCAL->buf,stream->mailbox,LOCAL->newmsg),
 		   O_RDWR,NIL)) >= 0) {
     fstat (*fd,&sbuf);		/* get current file size */
 				/* can we use this file? */
@@ -2295,7 +2308,7 @@ FILE *mix_data_open (MAILSTREAM *stream,int *fd,long *size,
       }
       close (*fd);		/* roll to a new file */
       while ((*fd = open (mix_file_data
-			  (LOCAL->buf,LOCAL->dir,
+			  (LOCAL->buf,stream->mailbox,
 			   LOCAL->newmsg = mix_modseq (LOCAL->newmsg)),
 			  O_RDWR | O_CREAT | O_EXCL,sbuf.st_mode)) < 0);
       *size = 0;		/* brand new file */
@@ -2323,14 +2336,15 @@ FILE *mix_sortcache_open (MAILSTREAM *stream)
   MESSAGECACHE *elt;
   SORTCACHE *sc;
   STRINGLIST *sl;
+  struct stat sbuf;
   int rdonly = NIL;
   FILE *srtcf = NIL;
   mailcache_t mc = (mailcache_t) mail_parameters (NIL,GET_CACHE,NIL);
+  fstat (LOCAL->mfd,&sbuf);
   if (!stream->nmsgs);		/* do nothing if mailbox empty */
-				/* open index file */
-  else if (((fd = open (LOCAL->sortcache,O_RDWR|O_CREAT,
-			(int) mail_parameters (NIL,GET_MBXPROTECTION,NIL)))<0)
-	   && !(rdonly = ((fd = open (LOCAL->sortcache,O_RDONLY,NIL)) >= 0)))
+				/* open sortcache file */
+  else if (((fd = open (LOCAL->sortcache,O_RDWR|O_CREAT,sbuf.st_mode)) < 0) &&
+	   !(rdonly = ((fd = open (LOCAL->sortcache,O_RDONLY,NIL)) >= 0)))
     MM_LOG ("Error opening mix sortcache file",WARN);
 				/* acquire lock and FILE */
   else if (!flock (fd,rdonly ? LOCK_SH : LOCK_EX) &&
@@ -2344,7 +2358,8 @@ FILE *mix_sortcache_open (MAILSTREAM *stream)
 				/* sequence changed from last time? */
   else if (i > LOCAL->sortcacheseq) {
     LOCAL->sortcacheseq = i;	/* update sequence */
-    while ((s = t = mix_read_record (srtcf,LOCAL->buf,LOCAL->buflen)) && *s &&
+    while ((s = t = mix_read_record (srtcf,LOCAL->buf,LOCAL->buflen,
+				     "sortcache")) && *s &&
 	   (msg = "uid") && (*s++ == ':') && isxdigit (*s)) {
       uid = strtoul (s,&s,16);
       if ((*s++ == ':') && isxdigit (*s)) {
@@ -2516,6 +2531,7 @@ FILE *mix_sortcache_open (MAILSTREAM *stream)
     fclose (srtcf);		/* so close it and return as if error */
     srtcf = NIL;
   }
+  else fchmod (fd,sbuf.st_mode);
   return srtcf;
 }
 
@@ -2583,27 +2599,38 @@ long mix_sortcache_update (MAILSTREAM *stream,FILE **sortcache)
  * Accepts: open FILE
  *	    buffer
  *	    buffer length
+ *	    record type
  * Returns: buffer if success, else NIL (zero-length buffer means EOF)
  */
 
-char *mix_read_record (FILE *f,char *buf,unsigned long buflen)
+char *mix_read_record (FILE *f,char *buf,unsigned long buflen,char *type)
 {
-  char *s;
-  buf[buflen-1] = '\0';		/* ensure string tied off */
+  char *s,tmp[MAILTMPLEN];
+				/* ensure string tied off */
+  buf[buflen-2] = buf[buflen-1] = '\0';
   while (fgets (buf,buflen-1,f)) {
-    if (!(s = strchr (buf,'\012'))) {
-      MM_LOG ("Oversize mix status record",ERROR);
+    if (s = strchr (buf,'\012')) {
+      if ((s != buf) && (s[-1] == '\015')) --s;
+      *s = '\0';		/* tie off buffer */
+      if (s != buf) return buf;	/* return if non-empty buffer */
+      sprintf (tmp,"Empty mix %s record",type);
+      MM_LOG (tmp,WARN);
+    }
+    else if (buf[buflen-2]) {	/* overlong record is bad news */
+      sprintf (tmp,"Oversize mix %s record: %.512s",type,buf);
+      MM_LOG (tmp,ERROR);
       return NIL;
     }
-    if ((s != buf) && (s[-1] == '\015')) --s;
-    *s = '\0';			/* tie off buffer */
-    if (s != buf) return buf;	/* return if non-empty buffer */
+    else {
+      sprintf (tmp,"Truncated mix %s record: %.512s",type,buf);
+      MM_LOG (tmp,WARN);
+      return buf;		/* pass to caller anyway */
+    }
   }
   buf[0] = '\0';		/* return empty buffer on EOF */
   return buf;
 }
-
-
+
 /* MIX read sequence record
  * Accepts: open FILE
  * Returns: sequence value, or NIL if failure
@@ -2613,7 +2640,7 @@ long mix_read_sequence (FILE *f)
 {
   unsigned long ret;
   char *s,tmp[MAILTMPLEN];
-  if (!mix_read_record (f,tmp,MAILTMPLEN-1)) return NIL;
+  if (!mix_read_record (f,tmp,MAILTMPLEN-1,"sequence")) return NIL;
   switch (tmp[0]) {		/* examine record */
   case '\0':			/* end of file */
     ret = 1;			/* start a new sequence regime */
